#import <objectbase/SwarmObject.h>
#import <space.h>


//Following needed to tell compiler classes exist
// We can't include Farmer.h and Acre.h here because it would create
// a circularity.
@class Farmer;
@class Acre;


//Just suppose there are 3 land uses
#define NUMTYPES 3


// Here is the FarmWorld object itself. It inherits very little behaviour,
// but contains various objects to store properties of the world.
@interface FarmWorld: SwarmObject
{
  unsigned int xsize, ysize;				  // grid size
  
   const char *soilTypeDataFile;			  // filename for max sugar
  id <Discrete2d> soilType;			  // max sugar at each spot
  int globalSoilTypeMax;			  // absolute maximum for space
  
  id <Grid2d> agentGrid;			  // agent positions
  id <Grid2d> landGrid;

}

- setSizeX: (int)x Y: (int)y;

- (int)getSizeX;

- (int)getSizeY;

- setSoilTypeDataFile: (const char *)s;


- updateWeather;

- calculateProductivity;

- (double)calculateProductionSoil: (int)s Product: (int)p Weather: (double)w;

- (int)getWeatherAtX: (int)x Y: (int)y;

- (int)getGlobalSoilTypeMax;

- (Farmer *)getAgentAtX: (int)x Y: (int)y;

- addAgent: (Farmer *)agent atX: (int)x Y: (int)y;

- (Farmer *)getOwnerAtX: (int)x Y: (int)y;
- (Acre *)claimOwnershipAgent: (Farmer *)agent atX: (int)x Y: (int)y;

- removeAgent: (Farmer *)agent;

// Get the array of all the agents in the space.
- (id <Grid2d>)getAgentGrid;
- (id <Grid2d>)getLandGrid;

// coordinate normalization. This handles coordinates out of range by
// wrapping them around.
- (int)xnorm: (int)x;
- (int)ynorm: (int)y;

@end
