
#import "FarmWorld.h"
#import "Farmer.h"
#import "Acre.h"

#import <simtools.h> // InFile
#import <misc.h> // strdup

@implementation FarmWorld

- createEnd
{
  [super createEnd];

  if (xsize <= 0 || ysize <= 0)
    [InvalidCombination raiseEvent:
                          "FarmWorld was created with an invalid size\n"];
  if (soilTypeDataFile == NULL)
    [InvalidCombination
      raiseEvent:
        "FarmWorld was created without a data file for soil type.\n"];
  
//    weather = [Discrete2d createBegin: [self getZone]];
//    [weather setSizeX: xsize Y: ysize];
//    weather = [weather createEnd];
   
 

#ifndef USE_HDF5  

   soilType = [Discrete2d createBegin: [self getZone]];
   [soilType setSizeX: xsize Y: ysize];
   soilType = [soilType createEnd];
   
  //the space library's setDiscrete2d:toFile: method returns the max value
   globalSoilTypeMax = [soilType setDiscrete2d: soilType toFile: soilTypeDataFile];

 
#else
  {
    int findGlobalMaxSoilType (id discrete2d)
      {
        int maxval;
        unsigned x, y;
        
        maxval = [discrete2d getValueAtX: 0 Y: 0];
        for (y = 0; y < ysize; y++)
          for (x = 0; x < xsize; x++)
            {
              int val = [discrete2d getValueAtX: x Y: y];
              
              if (val > maxval)
                maxval = val;
            }
        return maxval + 1;
      }
    
    soilType = [hdf5AppArchiver getObject: "soilTypeDiscrete2d"];
    if (!soilType)
      {
        soilType = [Discrete2d createBegin: [self getZone]];
        [soilType setSizeX: xsize Y: ysize];
        soilType = [soilType createEnd];
      }
    globalSoilTypeMax = findGlobalMaxSoilType (soilType);
  }
#endif

 

  agentGrid = [Grid2d createBegin: [self getZone]];
  [agentGrid setSizeX: xsize Y: ysize];
  agentGrid = [agentGrid createEnd];


  landGrid =  [Grid2d createBegin: [self getZone]];
  [landGrid setSizeX: xsize Y: ysize];
  landGrid = [landGrid createEnd];

  {
    unsigned x, y;
        
    for (y = 0; y < ysize; y++)
      for (x = 0; x < xsize; x++)
	{
	  Acre * anAcre = [Acre create: [self getZone]];
	  [anAcre setX: x Y: y];
	  [anAcre setSoilType: [soilType getValueAtX: x Y: y]];
	  [anAcre setWeather: 9.999];
	  [landGrid putObject: anAcre atX: x Y: y];
	}
  }
 
  return self;
}


// Just a silly example to show weather can be updated
- updateWeather
{
  unsigned int x, y;
  // loop through the world
  for (x = 0; x < xsize; x++)
    {
      for (y = 0; y < ysize; y++)
        {
	  Acre * anAcre = [landGrid getObjectAtX: x Y: y];
          double weatherHere = [anAcre getWeather];
          double randDouble = [uniformDblRand getDoubleWithMin: -weatherHere withMax: 5 ];
	  [anAcre setWeather: weatherHere + randDouble];
        }
    }
  
  return self;
}

//loop across each acre
- calculateProductivity
{
   unsigned int x, y;
  // loop through the world
  for (x = 0; x < xsize; x++)
    {
      for (y = 0; y < ysize; y++)
        {
	  Acre * anAcre = [landGrid getObjectAtX: x Y: y];
	  int soil = [anAcre getSoilType];
	  double theWeather = [anAcre getWeather];
	  int product = [anAcre getUseType];
	  double quantity = [self calculateProductionSoil: soil Product: product Weather: theWeather];
	  [anAcre setProduction: quantity];
	}
    }
  return self;
}

- (double)calculateProductionSoil: (int)s Product: (int)p Weather: (double)w
{
	  //TODO: need magic formula!
  return [uniformDblRand getDoubleWithMin: 0 withMax: 5 ];
}


- (int)getWeatherAtX: (int)x Y: (int)y
{
  x = [self xnorm: x];
  y = [self ynorm: y];
  return (int)[ [landGrid getObjectAtX: x Y: y] getWeather];
}


// Return the agent at a particular spot.
- (Farmer *)getAgentAtX: (int)x Y: (int)y
{
  return [agentGrid getObjectAtX: [self xnorm: x] Y: [self ynorm: y]];
}

// Add a new agent to the world. Note, this code updates the agent's
// own idea of where it is. We're violating OO encapsulation here,
// but that's ok.
- addAgent: (Farmer *)agent atX: (int)x Y: (int)y
{
  x = [self xnorm: x];
  y = [self ynorm: y];
  agent->x = x;
  agent->y = y;
  [agentGrid putObject: agent atX: x Y: y];

  return self;
}

// Return nil if the acre is unowned.
// Otherwise, return the owner.
- (Farmer *)getOwnerAtX: (int)x Y: (int)y
{
  Acre * anAcre = nil;
  
  Farmer * aFarmer = nil;

  x = [self xnorm: x];
  y = [self ynorm: y];

  anAcre = [landGrid getObjectAtX: x Y: y];
  aFarmer = [anAcre getOwner];


  if (aFarmer != nil)
    {
      fprintf(stderr,"====================getOwnerAtX:Y: called========================\n");
      fprintf(stderr,"Farmworld at %d, %d, Following you see the Acre's info :\n", x,y);
      [anAcre sayHi];
    }

  return aFarmer;
}


- (Acre *)claimOwnershipAgent: (Farmer *)agent atX: (int)x Y: (int)y
{
  Acre * anAcre;
  x = [self xnorm: x];
  y = [self ynorm: y];
  anAcre = [landGrid getObjectAtX: x Y: y];
  [anAcre setOwner: agent];
  return anAcre;
}


// Remove an agent from the world.
- removeAgent: (Farmer *)agent
{
  int x, y;

  x = [self xnorm: agent->x];
  y = [self ynorm: agent->y];
 
  [agent disclaimAllOwnership];

  if ([self getAgentAtX: x Y: y] == agent)
    [agentGrid putObject: nil atX: x Y: y];

  return self;
}



- setSizeX: (int)x Y: (int)y
{
  xsize = x;
  ysize = y;

  return self;
}

- (int)getSizeX
{
  return xsize;
}

- (int)getSizeY
{
  return ysize;
}

- (int)getGlobalSoilTypeMax
{
  return globalSoilTypeMax;
}

- (id <Grid2d>)getAgentGrid
{
  return agentGrid;
}

- (id <Grid2d>)getLandGrid
{
  return landGrid;
}



// Copy the filename into our object
- setSoilTypeDataFile: (const char *)s
{
  soilTypeDataFile = strdup (s);
  return self;
}

// normalize coordinates
- (int)xnorm: (int)x
{
  if (x < 0)					  // negative?
    return (x + xsize * 128) % xsize;		  // make positive, round
  else if (x >= (int)xsize)				  // too big? round.
    return x % xsize;
  else
    return x;					  // just right..
}

- (int)ynorm: (int)y
{
  if (y < 0)
    return (y + ysize * 128) % ysize;
  else if (y >= (int)ysize)
    return y % ysize;
  else
    return y;
}



@end
