#import "ModelSwarm.h"
#import "Output.h"
#import "Farmer.h"

#import <random.h>  //uniformUnsRand uniformIntRand

#include <misc.h> // strdup


@implementation ModelSwarm

+ createBegin: aZone
{
  ModelSwarm *obj;
  id <ProbeMap> probeMap;

  obj = [super createBegin: aZone];

  // Parameters for the simulation
  obj->numAgents = 400;

  obj->deathAgeMin = 99998;
  obj->deathAgeMax = 100000;
  obj->worldXSize = 50;
  obj->worldYSize = 50;
  obj->datafile = strdup("soiltypes.pgm");


  // The creation of a probe map - these parameters are the ones that
  // are easy to modify in the GUI.
  probeMap = [EmptyProbeMap createBegin: aZone];
  [probeMap setProbedClass: [self class]];
  probeMap = [probeMap createEnd];

  [probeMap addProbe: [probeLibrary getProbeForVariable: "numAgents"
				    inClass: [self class]]];
  //   [probeMap addProbe: [probeLibrary getProbeForVariable: "alpha"
  // 				    inClass: [self class]]];
  //   [probeMap addProbe: [probeLibrary getProbeForVariable: "replacement"
  // 				    inClass: [self class]]];
  //   [probeMap addProbe: [probeLibrary getProbeForVariable: "maxMetabolism"
  //				    inClass: [self class]]];
  //   [probeMap addProbe: [probeLibrary getProbeForVariable: "maxVision"
  //				    inClass: [self class]]];
  //   [probeMap addProbe: [probeLibrary getProbeForVariable: "minInitialSugar"
  //				    inClass: [self class]]];
  //   [probeMap addProbe: [probeLibrary getProbeForVariable: "maxInitialSugar"
  //				    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "deathAgeMin"
				    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "deathAgeMax"
				    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "datafile"
				    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForMessage: "addNewRandomAgent"
				    inClass: [self class]]];

  [probeLibrary setProbeMap: probeMap For: [self class]];
  
  return obj;
}

- createEnd
{
  agentCount=0;
  return [super createEnd];
}


- buildObjects
{
  int i;
  
  [super buildObjects];

  farmWorld = [FarmWorld createBegin: self];
  [farmWorld setSizeX: worldXSize Y: worldYSize];
  [farmWorld setSoilTypeDataFile: datafile];
  farmWorld = [farmWorld createEnd];

  agentList = [List create: self];

  // And create a bunch of agents to live in the world.
  for (i = 0; i < numAgents; i++)
    [self addNewRandomAgent];

  // Create a "reaper queue" to manage agent deaths
  reaperQueue = [List create: self];

  // And create a suffler object to randomize agent order
  shuffler = [ListShuffler createBegin: self];
  [shuffler setUniformRandom: uniformUnsRand];
  shuffler = [shuffler createEnd];

  output = [Output create: self];
  [output setModelSwarm: self];
  [output buildObjects];
  
  return self;
}

// Now it's time to create the schedule.
- buildActions
{

  id modelActions;  // holds together a sequence of actions for each timestep
  [super buildActions];

  modelActions = [ActionGroup create: self];
  //  [modelActions createActionTo: shuffler message: M(shuffleWholeList:) : agentList];

  [modelActions createActionForEach: agentList message: M(step)];

  [modelActions createActionTo: farmWorld message: M(updateWeather)];

  [modelActions createActionTo: farmWorld message: M(calculateProductivity)];

  [modelActions createActionTo: self message: M(reapAgents)];

  [modelActions createActionTo: output message: M(step)];

  // The schedule is just running our actions over and over again
  modelSchedule = [Schedule createBegin: self];
  [modelSchedule setRepeatInterval: 1];
  modelSchedule = [modelSchedule createEnd];
  [modelSchedule at: 0 createAction: modelActions];

  return self;
}

// Create a new agent at random and put it in the world.
// This allows a "LAND GRAB" where each new agent who
// finds a space can lay claim to spaces in the area
// I fiddled with values for the allowed radius of grabbing.
// Then each agent can reach left and right and right and up
// and down, but only a random number smaller than grabRadius.
// That makes it fun, generates some variety. Or whatever...
- addNewRandomAgent
{
  int x, y;
  Farmer *agent;
  id aThing;
  int grabRadius = 3;


  // Create the agent object
  agent = [Farmer createBegin: self];
  [agent setModelSwarm: self];
  agent = [agent createEnd];
 [agent setIDNumber: agentCount++];

  //Give the agent a random initial position and parameters. Find open spaces & land

  do 
    {
      x = [uniformIntRand getIntegerWithMin: 0 withMax: [farmWorld getSizeX]-1];
      y = [uniformIntRand getIntegerWithMin: 0 withMax: [farmWorld getSizeY]-1];
      aThing = [farmWorld getAgentAtX: x Y: y];
      if (aThing == nil)
	{
	  int i, j,  xx, yy;
	  [farmWorld addAgent: agent atX: x Y: y];
	  [agent addAcre: [farmWorld claimOwnershipAgent: agent atX: x Y: y]];
	  xx =  [uniformIntRand getIntegerWithMin: 0 withMax: grabRadius];
	  yy =  [uniformIntRand getIntegerWithMin: 0 withMax: grabRadius];
	  for ( i = 0; i < xx; i++)
	    {
	      for (j = 0; j < yy; j++)
		{
		  if ( [farmWorld getOwnerAtX: x+i Y: y+j] == nil)
		    {
		      Acre * anAcre = [farmWorld claimOwnershipAgent: agent atX: x+i Y: y+j];
		      [agent addAcre: anAcre];
		    }
		}
	    }
	  aThing = agent;
	}
    } while (aThing == nil);

  //  [agent setCurrentSugar: 
  //	   [uniformIntRand getIntegerWithMin: minInitialSugar
  //			   withMax: maxInitialSugar]];
  // [agent setMetabolism:
  //	   [uniformIntRand getIntegerWithMin: 1 withMax: maxMetabolism]];
  // [agent setVision: 
  //	   [uniformIntRand getIntegerWithMin: 1 withMax: maxVision]];
  
  [agent setDeathAge:
	   [uniformIntRand getIntegerWithMin: deathAgeMin
			   withMax: deathAgeMax]];
  
 
  [agentList addLast: agent];

 
  return self;
}


- agentDeath: (Farmer *)agent
{
  [reaperQueue addLast: agent];
  //  if (replacement)	  // Replacement rule R (p.32)
  //  [self addNewRandomAgent];
  return self;
}

// remove all the agents on the reaperQueue from the agentList
// This allows us to defer the death of an agent until it's safe to
// remove it from the list.
- reapAgents
{
  id index, agent;
  
  index = [reaperQueue begin: self];
  while ((agent = [index next]))
    {
      [agentList remove: agent];
      [agent drop];
    }
  [reaperQueue removeAll];
  return self;
}

// A technical detail of scheduling
- (id <Activity>)activateIn: swarmContext
{
  [super activateIn: swarmContext];
  [modelSchedule activateIn: self];
  return [self getActivity];
}

// accessor methods
- (FarmWorld *)getFarmWorld
{
  return farmWorld;
}

- getAgentList
{
  return agentList;
}


- (void)drop
{
  [output drop];
}

@end
