// This program is distributed without any warranty; without even the
// implied warranty of merchantability or fitness for a particular purpose.
// See file LICENSE for details and terms of copying.

#import "ObserverSwarm.h"

@implementation ObserverSwarm

// Set up configurable display parameters
//   how often to update the display,
+ createBegin: aZone
{
  ObserverSwarm *obj;
  id <ProbeMap> probeMap;
  
  obj = [super createBegin: aZone];

  obj->displayFrequency = 1;
  obj->parameterFile = NULL;

  probeMap = [EmptyProbeMap createBegin: aZone];
  [probeMap setProbedClass: [self class]];
  probeMap = [probeMap createEnd];

  [probeMap addProbe: [probeLibrary getProbeForMessage: "setParameterFile:"
 				    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForMessage: "saveParameters:"
				    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "displayFrequency"
				    inClass: [self class]]];

  [probeLibrary setProbeMap: probeMap For: [self class]];

  return obj;
}


- buildObjects
{
  [super buildObjects];
  
  CREATE_ARCHIVED_PROBE_DISPLAY (self);

 // Set our state to stopped - this code also waits until the user
  // clicks "go" so he has a chance to change parameters.
  [controlPanel setStateStopped];


  // If a parameter file was entered, use it. 
  if (parameterFile != NULL)
    {
      id archiver = [LispArchiver create: self setPath: parameterFile];
      

      if ((modelSwarm = [archiver getWithZone: self
                                            key: "model"]) == nil)
	raiseEvent(InvalidOperation,
		   "Can't find the parameters to create modelSwarm");
      [archiver drop];
    }
  else
    {
      modelSwarm = [ModelSwarm create: self];
    }
  // Now create probes for the Swarms so the user can retool settings
  CREATE_ARCHIVED_PROBE_DISPLAY (modelSwarm);
  
  // Stop the control panel again in case users want to revise
  // the settings.
  [controlPanel setStateStopped];

  
  // Now tell the model swarm to build its objects.
  [modelSwarm buildObjects];

 
  return self;
}  

- (char*)setParameterFile: (char*)aString
{
  char temp[100];
  sprintf (temp,"parameters/%s.scm",aString); 
  parameterFile = strdup (temp);
  return parameterFile;
}

      
- saveParameters: (char*)aString
 {
   char dataArchiveName[100];
   snprintf(dataArchiveName,100,"parameters/%s.scm",aString);
   id dataArchiver = [LispArchiver create: self setPath: dataArchiveName];
   
   [dataArchiver putShallow: "model" object: modelSwarm];
   [dataArchiver sync];
   [dataArchiver drop];
   return self;
 }



// The display schedule
- buildActions 
{
  [super buildActions];
  
  [modelSwarm buildActions];

  // The display schedule is just a list of actions in a row

  displayActions = [ActionGroup create: self];
  //  [displayActions createActionTo: self message: M(_updateDisplay_)];

  //[displayActions createActionTo: attributeGraph   message: M(step)];
//   if (drawPopulationGraph)
//     [displayActions createActionTo: populationGraph message: M(step)];
//   if (drawWealthHistogram)
//     [displayActions createActionTo: self message: M(_updateHistogram_)];

  [displayActions createActionTo: probeDisplayManager  message: M(update)];
  [displayActions createActionTo: actionCache          message: M(doTkEvents)];
#ifdef MAKEMOVIE
  [displayActions createActionTo: self                 message: M(writeFrame)];
#endif
  
  displaySchedule = [Schedule createBegin: self];
  [displaySchedule setRepeatInterval: displayFrequency];
  displaySchedule = [displaySchedule createEnd];
  [displaySchedule at: 0 createAction: displayActions];

  return self;
}  

// scheduling details. The model swarm is activated here.
- (id <Activity>)activateIn: swarmContext
{
  [super activateIn: swarmContext];
  [modelSwarm activateIn: self];
  [displaySchedule activateIn: self];
  return [self getActivity];
}

#ifdef MAKEMOVIE
- writeFrame
{
  char filename[256];
  id aPixmap;
  sprintf(filename, "%04ld.ppm", getCurrentTime());
  aPixmap = [Pixmap createBegin: self];
  [aPixmap setWidget: worldRaster];
  [aPixmap setDecorationsFlag: NO];
  aPixmap = [aPixmap createEnd];
  [aPixmap save: filename];
  [aPixmap drop];
  return self;
}
#endif


- (void)drop
{
  [modelSwarm drop];
}

@end
