#import "Output.h"

#include <misc.h> // stdio, time
//#import <time.h>
#import "ModelSwarm.h"
#import "Parameters.h"

#import "Farmer.h"
#import "Acre.h"
#import "FarmWorld.h"
#import <gui.h> //guessing to see about errors in gui


#define NUMBINS 100

@implementation Output


- createEnd
{
  //  id < CustomProbeMap >  probeMap;
  
//   probeMap =
//     [CustomProbeMap create: [self getZone]
//                     forClass: [self class]
//                     withIdentifiers: ":", "showHistoDist:",
// 		    "dropParty:", 
// 		    "mergeParties:And:",
// 		    "addNewPartyAt:",
// 		    NULL];

 
//   [probeLibrary setProbeMap: probeMap For: [self class]];
  CREATE_ARCHIVED_PROBE_DISPLAY (self);
  return self;
}


//Create things needed for writing data.  Make sure file names are set up.
- buildObjects
{
  dataFileExists = NO;
  graphsExist = NO;
  [self createTimeString];

  [self createGraphWriter];
 
  [self prepareCOutputFile];

  if (swarmGUIMode) [self createRasters];

  return self;
}


- (void)setModelSwarm: aSwarm
{
  modelSwarm = aSwarm;
}


- (void)createTimeString
{
  int i;
  time_t runTime; /*"Return from the system's time() function"*/
  int runNumber = getInt((Parameters *)arguments,"run"); 
 

  if ( runNumber == -1)
    {
      runTime = time(NULL);
      strcpy (timeString, ctime(&runTime));
      //this scans through and converts spaces and colons to underscores.
      for (i = 0; i < 100; i++)
	{
	  if (timeString[i] == ' ' || timeString[i] == ':')
	    timeString[i] = '_';
	  else if (timeString[i] == '\n')
	    timeString[i] = '\0';
	}
    }
  else
    {
      
      snprintf (timeString, 100, "%4d", runNumber);
      for (i = 0; i < 100; i++)
	{
	  if (timeString[i] == ' ' || timeString[i] == ':')
	    timeString[i] = '0';
	  else if (timeString[i] == '\n')
	    timeString[i] = '\0';
	  
	}
    }
  
}



// Create an EZGraph object. This ALWAYS writes data into a file, if 
// you are either in GUI or batchmode. Only if you are in GUI mode
// will it draw a graph to the screen, however.
- (void)createGraphWriter
{
  char hdfEZGraphName[100];

  id <HDF5> hdf5container; /*"HDF5 data container"*/
  int i;

  const char *colors[] =
  {"black", "brown", "red", "green", "blue", "orange", "purple", "DeepPink","grey50"};


  strcpy (hdfEZGraphName,"output_");
  strcat (hdfEZGraphName, timeString);
  strcat (hdfEZGraphName, ".hdf");



  hdf5container = [HDF5 createBegin: [self getZone]];
  [hdf5container setWriteFlag: YES];
  [hdf5container  setName: hdfEZGraphName];
  hdf5container = [hdf5container createEnd];
    
  graphsExist = YES;

  populationGraph = [EZGraph createBegin: [[self getZone] getZone]];
  SET_WINDOW_GEOMETRY_RECORD_NAME (populationGraph);

  if (swarmGUIMode == YES)
    {
      [populationGraph setGraphics: YES];
    }
  else
    {
      [populationGraph setGraphics: NO];
    }

  [populationGraph setTitle: "Use types over time"];
  [populationGraph setAxisLabelsX: "time" Y: "proportion"];
  [populationGraph setColors: colors count: 9];
  populationGraph = [populationGraph createEnd];
      
  
  [populationGraph enableDestroyNotification: self
                 notificationMethod: @selector (_populationGraphDeath_:)];
 
  //One sequence for each type
  popGraphSequences= [Array create: [self getZone] setCount: NUMTYPES];


  for (i=0; i < NUMTYPES; i++)
    {
      id aSeq;
      char aString[15];
      snprintf(aString,15,"Type %d", i);

      aSeq = [populationGraph createAverageSequence: aString
			      withFeedFrom: [modelSwarm getAgentList]
			      andSelector: M(getProportionType:)];
      [aSeq setUnsignedArg: i];
      [popGraphSequences atOffset: i put: aSeq];
    }


    
  if (swarmGUIMode == YES)
    {
      // Create a histogram of agent wealth distribution
      wealthHistogram = [EZBin createBegin: [self getZone]];
      SET_WINDOW_GEOMETRY_RECORD_NAME (wealthHistogram);
      [wealthHistogram setTitle: "Distribution of Land Ownership"];
      [wealthHistogram setAxisLabelsX: "acres owned" Y: "number of agents"];
      [wealthHistogram setBinCount: 25];
      [wealthHistogram setLowerBound: 0];
      [wealthHistogram setUpperBound: 50];
      [wealthHistogram setCollection: [modelSwarm getAgentList]];
      [wealthHistogram setProbedSelector: M(getAcreage)];
      wealthHistogram = [wealthHistogram createEnd];
      [wealthHistogram enableDestroyNotification: self
                       notificationMethod: @selector(_wealthHistogramDeath_:)];
    }
    
  
  
}


- createRasters
{

  int i;
  int  soilTypeValue = [[modelSwarm getFarmWorld] getGlobalSoilTypeMax];

  fprintf(stderr,"SoilTypeValue %d \n," , soilTypeValue);


  colormap = [Colormap create: [self getZone]];
  // Colours - shades of yellow for sugar values
  for (i = 0; i < soilTypeValue; i++)
    [colormap setColor: i
  	      ToRed: (double) i / (soilTypeValue - 1.0)
  	      Green: (double) i / (soilTypeValue - 1.0)
  	      Blue: 0];
 

  // Red for agents. Assigned to the number 100 - the agent needs to
  // know this magic number.
  [colormap setColor: 100 ToName: "red"];	  // agent colour
  [colormap setColor: 101 ToName: "black"];	  // agent colour
  
  // Next, create a 2d window for displaying the world.
  worldRaster = [ZoomRaster createBegin: [self getZone]];
  SET_WINDOW_GEOMETRY_RECORD_NAME (worldRaster);
  worldRaster = [worldRaster createEnd];
  [worldRaster enableDestroyNotification: self
               notificationMethod: @selector (_worldRasterDeath_:)];
  [worldRaster setColormap: colormap];
  [worldRaster setZoomFactor: 6];
  [worldRaster setWidth: [[[modelSwarm getFarmWorld] getAgentGrid] getSizeX]
               Height: [[[modelSwarm getFarmWorld] getAgentGrid] getSizeY]];
  [worldRaster setWindowTitle: "SugarScape"];
  [worldRaster pack];                             // draw the window.

  agentDisplay = [Object2dDisplay createBegin: [self getZone]];
  [agentDisplay setDisplayWidget: worldRaster];
  // [agentDisplay setDiscrete2dToDisplay: [farmWorld getAgentGrid]];
  //  [agentDisplay setObjectCollection: [modelSwarm getAgentList]];
  [agentDisplay setDiscrete2dToDisplay: [[modelSwarm getFarmWorld] getLandGrid]];
  [agentDisplay setDisplayMessage: M(drawSelfOn:)];   // draw method
  agentDisplay = [agentDisplay createEnd];


  // Enable probes on the world.
  [worldRaster setButton: ButtonRight
               Client: agentDisplay
               Message: M(makeProbeAtX:Y:)];


  return self;
}



- prepareCOutputFile
{
  if (dataFileExists == YES) return self;

  else
    {

      {
	char outputFileName[256];
	snprintf (outputFileName,256, "output_%s.data",timeString);
	
	if(!(outputFile = fopen(outputFileName,"w")))
	  abort();
	
	fprintf(outputFile,"run time  \n");
      }
      
      
      dataFileExists = YES;
    }
  return self;
}


// This method dumps out measures into a text file
- writeCDataRun: (int)run Time: (long)theTime
{


  
  [self printResults: run Time: theTime];

  return self;
}


- printResults: (int)run Time: (long unsigned)theTime
{
  
  fprintf(outputFile, "%d  %lu ", run, theTime);    
  fprintf(outputFile,"\n");
  
  
  fflush (outputFile);
  
  return self;
}




- step
{

  int run = getInt(arguments,"run");

  long simTime = getCurrentTime();

  //records the time in the Parameters object, in case we want
  //to refer to it later
 
  [(Parameters *)arguments setSimTime: simTime]; 
 
  if (!graphsExist) [self createGraphWriter];
  
  [populationGraph step];
 
  if (swarmGUIMode == YES)
    {
      [wealthHistogram reset];
      [wealthHistogram update];
      [wealthHistogram output];

      [weatherDisplay display];
      [agentDisplay display];
      [worldRaster drawSelf];
    }

  [self writeCDataRun: run Time: simTime];
  return self;
}  


- (BOOL)checkToStop
{
  static int emptySteps;
  if ( 0 )
    {
      emptySteps++;
    }					   
  else
    {
      emptySteps = 0;
    }
  if (emptySteps > 10) return YES;
  return NO;
}


/*"It is necessary to drop the data writing objects in order to make
sure they finish their work.
"*/
- (void)drop 
{
  if (populationGraph) [populationGraph drop];
  if (wealthHistogram)  [wealthHistogram drop];
  [super drop];
}



- _populationGraphDeath_ : caller
{
  [populationGraph drop];
  populationGraph = nil;
  return self;
}


- _wealthHistogramDeath_ : caller
{
  [wealthHistogram drop];
  wealthHistogram = nil;
  return self;
}



@end



