#import <objectbase.h>

#import "Parameters.h"

#import <misc.h>  //for atoi
#import <swarm.h> // for swarmGUIMode



id
makeProbe (id obj, const char *ivarName)
{
  id probe = [VarProbe createBegin: [obj getZone]];
  [probe setProbedClass: [obj getClass]];
  [probe setProbedVariable: ivarName];
  return [probe createEnd];
}


int
getInt (id obj, const char *ivarName)
{
  id probe = makeProbe (obj, ivarName);
  int ret = [probe probeAsInt: obj];
  [probe drop];
  return ret;
}


double
getDbl (id obj, const char *ivarName)
{
  id probe = makeProbe (obj, ivarName);
  double ret = [probe probeAsDouble: obj];
  [probe drop];
  return ret;
}



@implementation Parameters

+ createBegin: aZone
 {
   static struct argp_option options[] = {
     {"inputfile", 'I',"filename",0,"set fn",5},
     {"run",'R',"RunNumber",0,"Run is...",6},
     {"threshold",'h',"threshold",0,"threshold (pct)",7},
     { 0 }
   };

   Parameters *obj = [super createBegin: aZone];
   [obj addOptions: options];

   
   obj->run = -1; /*time(0)*/
   obj->duration = 5000;
   obj->threshold = 5.00;
   obj->simTime = 0;
   return obj;
 }



- init
{
  id probeMap;
  probeMap = [EmptyProbeMap createBegin: [self getZone]];
  [probeMap setProbedClass: [self class]];
  probeMap = [probeMap createEnd];
  // Add in a bunch of variables, one per simulation parameter
   [probeMap addProbe: [probeLibrary getProbeForVariable: "threshold"
				     inClass: [self class]]]; 
 
   [probeMap addProbe: [probeLibrary getProbeForVariable: "run"
				     inClass: [self class]]];
 
   [probeMap addProbe: [probeLibrary getProbeForVariable: "duration"
				     inClass: [self class]]];
   
   [probeLibrary setProbeMap: probeMap For: [self class]];
   
   return self;
}



- (int)parseKey: (int) key arg: (const char*) arg
 {
   if (key == 'D')
     {
       if(arg)
	 duration = atoi(arg);
       return 0;
     }
   
   
   else if (key == 'R')
     {
       if(arg)
	 run = atoi(arg);
       return 0;
     }


   else if (key == 'h')
     {
       if(arg)
	 threshold = strtod(arg,0);
       return 0;
     }
   
   else
     return [super parseKey: key arg: arg];
 }


- (void)setSimTime: (long)x
{
  simTime = x;
}


- (long)getSimTime
{
  return simTime;
}

@end










